from dataclasses import dataclass, field

from typing import Any,Optional,List
from entsoe.entsoe import Area


import pandas as pd
import pytz

def _country_code_to_emoji(country_code):
    OFFSET = 127397
    return ''.join(chr(ord(c) + OFFSET) for c in country_code.upper())

@dataclass
class Country:
    capital: str
    country_code: str
    country_name: str
    UTC_time_difference: int = field(init=False)
    
    def calculate_utc_time_difference(self) -> Optional[pd.Timedelta]:
        try:
            print ("calculating timezone differecne")
            timezone = f"Europe/{self.capital}"
            local_tz = pytz.timezone(timezone)
            utc_dt = pd.Timestamp.utcnow().to_pydatetime()
            local_dt = utc_dt.astimezone(local_tz)
            UTC_time_difference = (local_dt.utcoffset().total_seconds()) // 3600
            delta = pd.Timedelta(hours=UTC_time_difference)
            return delta,UTC_time_difference
        
        except:
            print(f"{self.capital} not found in European capitals.")
            return None

    def __post_init__(self: Any) -> None:
        self.delta,self.UTC_time_difference = self.calculate_utc_time_difference()
        self.emoji = _country_code_to_emoji(self.country_code)

def get_country_by_code(countries: List[Country], country_code: str) -> Optional[Country]:
    """
    Returns the country object that matches the given country code.
    If no such object is found, returns None.
    """
    for country in countries:
        if country.country_code == country_code:
            return country
    return None

def get_country_by_name(countries: List[Country], country_name: str) -> Optional[Country]:
    """
    Returns the country object that matches the given country name.
    If no such object is found, returns None.
    """
    for country in countries:
        if country.country_name == country_name:
            return country
    return None

Austria = Country(capital="Vienna", country_code="AT", country_name="Austria")
Belgium = Country(capital="Brussels", country_code="BE", country_name="Belgium")
Bulgaria = Country(capital="Sofia", country_code="BG", country_name="Bulgaria")
Croatia = Country(capital="Zagreb", country_code="HR", country_name="Croatia")
Cyprus = Country(capital="Nicosia", country_code="CY", country_name="Cyprus")
Czech_republic = Country(capital="Prague", country_code="CZ", country_name="Czech Republic")
Denmark = Country(capital="Copenhagen", country_code="DK", country_name="Denmark")
Estonia = Country(capital="Tallinn", country_code="EE", country_name="Estonia")
Finland = Country(capital="Helsinki", country_code="FI", country_name="Finland")
France = Country(capital="Paris", country_code="FR", country_name="France")
Germany = Country(capital="Berlin", country_code="DE", country_name="Germany")
Greece = Country(capital="Athens", country_code="GR", country_name="Greece")
Hungary = Country(capital="Budapest", country_code="HU", country_name="Hungary")
Ireland = Country(capital="Dublin", country_code="IE", country_name="Ireland")
Italy = Country(capital="Rome", country_code="IT", country_name="Italy")
Latvia = Country(capital="Riga", country_code="LV", country_name="Latvia")
Lithuania = Country(capital="Vilnius", country_code="LT", country_name="Lithuania")
Luxembourg = Country(capital="Luxembourg", country_code="LU", country_name="Luxembourg")
Netherlands = Country(capital="Amsterdam", country_code="NL", country_name="Netherlands")
Poland = Country(capital="Warsaw", country_code="PL", country_name="Poland")
Portugal = Country(capital="Lisbon", country_code="PT", country_name="Portugal")
Romania = Country(capital="Bucharest", country_code="RO", country_name="Romania")
Slovakia = Country(capital="Bratislava", country_code="SK", country_name="Slovakia")
Slovenia = Country(capital="Ljubljana", country_code="SI", country_name="Slovenia")
Spain = Country(capital="Madrid", country_code="ES", country_name="Spain")
Sweden = Country(capital="Stockholm", country_code="SE", country_name="Sweden")

countries = [Austria, Belgium, Bulgaria, Croatia, Cyprus, Czech_republic, Denmark, Estonia, Finland, France, Germany, Greece, Hungary, Ireland, Italy, Latvia, Lithuania, Luxembourg, Netherlands, Poland, Portugal, Romania, Slovakia, Slovenia, Spain, Sweden]




def return_All_Areas(filterstring:str ="MBA"):
        
    result = []
    for area in Area:
        #print (area.name, area.meaning)
        if filterstring in area.meaning:
            result.append(area)
            print (area.name)

    return result




import spotON_sdk




print (spotON_sdk.Markets())
print (spotON_sdk.Markets().get_Market_by_area_code("AT").country.emoji)

for country in spotON_sdk.Markets().markets_List:
    print (country.name)




from dataclasses import dataclass
from typing import List

from ..constants.markets import Market
class Switchtypes():
    switch = "Switch"
    dimmer = "Dimmer"


class PeriodTypes():
    wholeDay :str = "24h Hour Period"
    timeFrame :str = "Timeframe"

@dataclass
class TimeFrameValidation():
    passed :bool
    def return_Validation_Result(self):
        if self.passed == False:
            return "<br> 🚨 <mark> Your Timeframe is SHORTER than your desired ON Time</mark> "
        else:
            return ""


@dataclass
class Config():
    switchtype :Switchtypes
    nr_of_Hours_On :int
    uninterrupted :bool
    market : Market 
    timeframe :List[List[int]] 
    periodType : str = ""
    bestHour :str = ""
    week: int = 100
    timeFrameValidation = TimeFrameValidation(False)

    def __post_init__(self):
        def check_nr_of_Hours_not_0():
            if self.nr_of_Hours_On < 1:
                raise ValueError(f"Hey your Number of Hours On can't be zero")

        def check_TimeFrame_is_large_enogh():
            if len(self.find_Possible_Hours()) > self.nr_of_Hours_On:
                self.timeFrameValidation = TimeFrameValidation(True)
            else:
                self.timeFrameValidation = TimeFrameValidation(False)
        
        def checkLogic():
            if self.uninterrupted:
                if len(self.timeframe) > 1:
                    raise ValueError(f"Hey your Timeframe has the Attribute of Beeing uninterruped means you can't have multiple timeframes")
            if self.switchtype == Switchtypes.switch and self.nr_of_Hours_On == 24:
                raise ValueError(f"Hey if it Should be on for {self.nr_of_Hours_On} there is no need for switching")


        def check_Overlapping_Arrays():
            possible_Hours = self.find_Possible_Hours()
            if(len(set(possible_Hours)) != len(possible_Hours)):
                raise ValueError(f"Hey your Timeframes {self.timeframe} have overlapping Hours. Reduce it to one")
            
        
        check_nr_of_Hours_not_0()
        check_TimeFrame_is_large_enogh()
        checkLogic()
        #check_Overlapping_Arrays()
        
    def find_Possible_Hours(self):
        possible_Hours = []
        for timeframe in self.timeframe:
            start,end = timeframe[0],timeframe[1]
            def loop_Hours(_start,_end):
                for hour in range (_start,_end):
                    possible_Hours.append(hour)
                
            if start < end:
                loop_Hours(start,end)
            elif start > end :  # timeframe goes over Midnight
                loop_Hours(start,25)
                loop_Hours(0,end)

        if possible_Hours == []:    # If its the same start and end consider the whole day is OK
            possible_Hours = [*range(0,24)]


        return possible_Hours




import pytz
import pandas as pd
from entsoe import EntsoePandasClient
from entsoe.mappings import Area
from datetime import datetime


from spotON_sdk import Country,Market


'''from mySecrets import mySecrets
client = EntsoePandasClient(api_key=mySecrets.ENTSOE_KEY)'''

def get_Time_in(timezone:str):
    # set the timezone for Lisbon
    local_tz = pytz.timezone(timezone)

    # get the current date and time in Lisbon timezone
    now_local_tz = pd.Timestamp.now(tz=local_tz)

    # set the time to 0:00
    today_0am = now_local_tz.floor('D')
    return today_0am

def timezone_difference(tz1, tz2 = Area.AT.tz, date=None):
    if date is None:
        date = datetime.now()

    timezone1 = pytz.timezone(tz1)
    timezone2 = pytz.timezone(tz2)

    tz1_offset = timezone1.utcoffset(date)
    tz2_offset = timezone2.utcoffset(date)

    timedelta = tz2_offset - tz1_offset
    return pd.Timedelta(timedelta)


def getTimeStamps(country_code,tomorrow,overrideTime):
    
    tomorrow_timestamp = tomorrow.strftime("%Y%m%d")

    timestamp_start = pd.Timestamp(tomorrow_timestamp,tz=country_code.tz)

    timestamp_end = timestamp_start + pd.Timedelta(hours=23)

    CET_Difference = timezone_difference(country_code.tz) 
    timestamp_start = timestamp_start - CET_Difference

    start = pd.Timestamp(timestamp_start)
    end = pd.Timestamp(timestamp_end)

    # Override Start
    if overrideTime:
        start = pd.Timestamp(overrideTime,tz=country_code.tz)

    return start,end,CET_Difference

def query_Data(country_code,client):
    today = get_Time_in(country_code.tz)
    tomorrow = today + pd.Timedelta(days=1)
    tomorrow = today


    start,end,CET_Difference = getTimeStamps(country_code,tomorrow)
    print (f"{start=}   {end=}  {CET_Difference=}")
    series = client.query_day_ahead_prices(country_code,start=start,end=end)
    return series,CET_Difference

def create_Dataframe(s:pd.Series,country_code):
    # Convert series to dataframe
    df = s.to_frame()

    # Rename the column
    df.columns = ['value']

    # Set the index to be a DatetimeIndex
    df.index = pd.DatetimeIndex(df.index,tz=country_code.tz)
    df.index = df.index 

    return df

def query_Data(country_code,client,overrideTime :str|None = None,today_instead_of_tomorrow= False):
    today = get_Time_in(country_code.tz)
    tomorrow = today + pd.Timedelta(days=1)

    if today_instead_of_tomorrow:
        tomorrow = today

    start,end,CET_Difference = getTimeStamps(country_code,tomorrow,overrideTime)
    
    print (f"{start=}   {end=}  {CET_Difference=}")
    series = client.query_day_ahead_prices(country_code,start=start,end=end)
    return series,CET_Difference

def create_Dataframe(s:pd.Series,market:Market):

    # Convert series to dataframe
    df = s.to_frame()

    # Rename the column
    df.columns = ['value']

    # Set the index to be a DatetimeIndex
    df.index = pd.DatetimeIndex(df.index,tz=market.area.tz)
    df.index = df.index 
    df["Area_Code"] = market.area_code

    return df




from dataclasses import dataclass
from entsoe import Area
import pandas as pd

import pandas as pd
from dataclasses import dataclass, field
from typing import Optional, Dict,Any
import pytz

from typing import List
from .countries import *
from .bidding_zones import bidding_zones

@dataclass
class Market():
    area:Area
    country :Country
    area_code :str = field(init=False)
    country_code : str = field(init=False)
    region_code : str | None = field(init=False,default=None)
    cities : str = field(init=False)

    def __post_init__(self):
        self.area_code = self.area.name
        country_region = self.area_code.split("_")
        self.country_code = country_region[0]
        if len(country_region) >=2:
            self.region_code = country_region[1]
        print (self.area_code)
        try:
            city_List = bidding_zones[self.area_code]["cities"]
            my_string = ', '.join(city_List)
            self.cities = my_string
        except:
            self.cities = ""


        self.name = f"{self.country.emoji} {self.country.country_name} {self.area_code} {self.cities}"
        #self.get_Market_by_area_code(self.area.name)

dataclass
class Markets():
    austria = Market(Area.AT,Austria)
    germany = Market(Area.DE_LU,Germany)
    luxembourg = Market(Area.DE_LU,Luxembourg)
    sweden1 = Market(Area.SE_1,Sweden)
    sweden2 = Market(Area.SE_2,Sweden)
    sweden3 = Market(Area.SE_3,Sweden)
    sweden4 = Market(Area.SE_4,Sweden)
    markets_List = [value for key, value in vars().items() if isinstance(value, Market)]

    @staticmethod
    def get_Market_by_area_code(area_code: str) -> Optional[Market]:
        for country in Markets.markets_List:
            if country.country_code == area_code:
                return country

        return None


def return_All_Areas(filterstring:str ="MBA"):
        
    result = []
    for area in Area:
        #print (area.name, area.meaning)
        if filterstring in area.meaning:
            result.append(area)
            print (area.name)

    return result


